;****************************************************************
;* Filename: MSDOSTIM/ASM					*
;* Rev Date: 10 Jan 97						*
;* Revision: 01.03.00						*
;****************************************************************
;* Copyright 1997-98 Peter W. Cervasio				*
;* Permission to distribute this in source and executable form  *
;* is hereby granted.  All other rights reserved.		*
;****************************************************************
;* TRS-80 Model 4 TRSDOS/LS-DOS program for setting the date	*
;* and time in Jeff Vavasour's Model 4 emulator by using the	*
;* MSDOS date and time functions.  This is much easier than	*
;* entering the date and time from the keyboard every time the	*
;* emulator is rebooted.					*
;****************************************************************

;
;----------------------------------------------------------------
;	Operating System equates
;----------------------------------------------------------------
;
@ABORT	EQU	15H		; Abort to DOS
@EXIT	EQU	16H		; Exit to DOS normally
@DATE	EQU	12H		; Get system date
@TIME	EQU	13H		; Get system time
@FLAGS	EQU	65H		; Get system flags
@DSPLY	EQU	0AH		; Display string at HL
@PARAM	EQU	11H		; Parse command line
@CKBRKC	EQU	6AH		; Check/reset BREAK bit
@HEXDEC	EQU	97		; 16 bit binary to ASCII
@DECHEX	EQU	96		; ASCII to binary
;
;----------------------------------------------------------------
;	Miscellaneous constants
;----------------------------------------------------------------
;
LF	EQU	0AH		; Line Feed
CR	EQU	0DH		; Carriage Return
ETX	EQU	03H		; End of text, no CR
;
	ORG	3000H
;
;----------------------------------------------------------------
;	Start the program
;----------------------------------------------------------------
;
START	PUSH	HL		; Save cmd line just in case
	DB	0EDH,20H,01H	; See if we're in the emulator
	JR	DOCLOCK		; Only happens if in it
;
;	Not in the emulator.  Show error and exit.
;
	LD	HL,ERMSG$	; Point to error message
	LD	A,@DSPLY	; Display message
	POP	HL		; Clean up the stack
	LD	HL,0		; Set no error code
	RET			;   and get out
;
;----------------------------------------------------------------
;	If we got past that, we're running in the emulator.
;----------------------------------------------------------------
;
DOCLOCK	POP	HL		; Get cmd line back
	LD	A,@CKBRKC	; Get break flag
	RST	28H
	JR	Z,CLOCK01	; Jump if it wasn't set
	LD	HL,0FFFFH	; Set errorlevel for return
	RET			;   to TRSDOS
;
CLOCK01	LD	A,(HL)		; See if any command line stuff was added
	CP	0DH		; Carriage return?
	JR	NZ,DOREAD	; Jump if it wasn't
	LD	A,0FFH
	LD	(GOTPRM),A	; Set flag that we have cmd line
	LD	HL,WELMSG$	; Sign on
	LD	A,@DSPLY
	RST	28H
;
;	Get the MSDOS time and date
;
DOREAD	LD	A,0
	DB	0EDH,021H,02AH	; do MSDOS function 2Ah
;
;	BC (CX)=Year, D (DH)=Month, E (DL)=Day
;
	LD	A,E
	LD	(DAY),A
	LD	A,D
	LD	(MONTH),A
	PUSH	BC
	POP	HL		; Move year to HL
	LD	DE,1900
	AND	A
	SBC	HL,DE
	LD	A,L
	LD	(YEAR),A
;
	LD	A,0
	DB	0EDH,021H,02CH	; MSDOS function 2CH
;
;	B (CH)=Hour, C (CL)=Minutes, D (DH)=Seconds
;
	LD	A,B
	LD	(HOUR),A
	LD	A,C
	LD	(MINUTE),A
	LD	A,D
	LD	(SECOND),A
;
;	Set TRSDOS/LS-DOS time
;
SETDOS	LD	HL,TIME$	; Location for current time
	LD	A,@TIME		; Get time and ptr to sys time
	RST	28H
	PUSH	DE		; Pointer to system time bytes
	POP	IY		; Make IY point to it
	LD	A,(HOUR)
	LD	(IY+2),A	; Set new system hours
	LD	A,(MINUTE)
	LD	(IY+1),A	; Set new system minutes
	LD	A,(SECOND)
	LD	(IY),A		; Set new system seconds
	LD	HL,TIME$	; Location for current time
	LD	A,@TIME		; Get time into string
	RST	28H
;
	LD	HL,DATE$	; Location for date string
	LD	A,@DATE		; Get date & ptr to sys date
	RST	28H
	PUSH	DE		; Move pointer to IY
	POP	IY
	LD	A,(MONTH)
	LD	(IY+2),A	; Set new system month
	LD	A,(DAY)
	LD	(IY+1),A	; Set new sytem date
	LD	A,(YEAR)
	LD	(IY),A		; Set new sytem year
	LD	HL,DATE$	; Location for date string
	LD	A,@DATE
	RST	28H

	LD	A,(GOTPRM)	; Any command line?
	OR	A
	JR	Z,PROGEND
	LD	HL,DATSET$	; Show the date/time
	LD	A,@DSPLY
	RST	28H
;
;----------------------------------------------------------------
;	End of the program
;----------------------------------------------------------------
;
PROGEND	LD	A,@EXIT		; Exit to TRSDOS
	LD	HL,0
	RST	28H		; Return to TRSDOS ready
;
;----------------------------------------------------------------
; Data storage locations
;----------------------------------------------------------------
;
YEAR	DB	0
MONTH	DB	0
DAY	DB	0
HOUR	DB	0
MINUTE	DB	0
SECOND	DB	0
;
WELMSG$ DB	'MSDOSTIM - Version 1.03 - Date/Time Setting utility',LF
	DB	'Copyright 1997-98, Peter W. Cervasio',LF,CR
;
ERMSG$	DB	'Requires Emulator version 2.3 or higher.',00H
;
DATSET$ DB	'System set to: '
DATE$	DB	'00/00/00 '
TIME$	DB	'00:00:00',CR
;
GOTPRM	DB	0
READCLK	DW	0               ; Dummy location...
;
	END	START

